;EnableExplicit

;MeterGadget is a mix of a gadget and a image, a few procedures have been made to make
;gadget handling easier. In general a MeterGadget behaves almost like a normal ImageGadget.

;Otherwise normal gadget functions can be used (that works with ImageGadget)
;and obviously since it is a imagegadget it has the same gadget events returned as well.
;Tooltips can be set just like with any other ImageGadget.
;flags constants are the same as the flags constants used for ImageGadget like #PB_Image_Border

;IMPORTANT! SetGadgetData() MUST NOT be used with the MeterGadget,
;it's used for the structure pointers.
;Use SetMeterGadgetData() and GetMeterGadgetData() instead.

Structure MeterGadget_struct
 image.l
 imageid.l
 x.l
 y.l
 w.l
 h.l
 bgcolor.l
 fgcolor.l
 linecolor.l
 offset.l
 userdata.l
 flags.l
 table.l[1024]
EndStructure


;Create #Image and #Gadget etc.
Procedure.l MeterGadget(gadget.l,x.l,y.l,width.l,height.l,bgcolor.l,fgcolor.l,linecolor.l,flags.l=#Null)
 Protected *m.MeterGadget_struct,result.l=#Null,n.l,p.l=0,newgadget.l
 If IsGadget(gadget)
  *m=GetGadgetData(gadget)
  If *m
   result=*m\image
  EndIf
 EndIf
 *m=AllocateMemory(SizeOf(MeterGadget_struct))
 If *m
  *m\flags=flags
  If *m\flags=#PB_Image_Border
   *m\w=width-2
   *m\h=height-2
  Else
   *m\w=width
   *m\h=height
  EndIf
  *m\x=x
  *m\y=y
  *m\bgcolor=bgcolor
  *m\fgcolor=fgcolor
  *m\linecolor=linecolor
  *m\image=CreateImage(#PB_Any,width,height,#PB_Image_DisplayFormat)
  If IsImage(*m\image)
   *m\imageid=ImageID(*m\image)
   If StartDrawing(ImageOutput(*m\image))
    Box(p,p,*m\w,*m\h,*m\bgcolor)
    For n=0 To *m\h Step 10
      LineXY(p,n,*m\w,n,*m\fgcolor)
    Next
    For n=0 To width Step 10
      LineXY(n,p,n,*m\h,*m\fgcolor)
    Next
    StopDrawing()
    result=ImageGadget(gadget,*m\x,*m\y,*m\w,*m\h,*m\imageid,flags)
   EndIf
  EndIf
  If gadget=#PB_Any
   newgadget=result
  Else
   newgadget=gadget
  EndIf
  If IsGadget(newgadget)
   SetGadgetData(newgadget,*m)
  Else
   If IsImage(*m\image)
    FreeImage(*m\image)
   EndIf
   FreeMemory(*m)
  EndIf
 EndIf
 ProcedureReturn result
EndProcedure

;Use this instead of SetGadgetState()
Procedure SetMeterGadgetState(gadget.l,state.l)
 Protected *m.MeterGadget_struct,result.l
 Protected n.l,x1.l,x2.l,y1.l,y2.l,bottom.l,percent.f,h.l,p.l
 *m=GetGadgetData(gadget)
 If *m\offset=10
   *m\offset=0
 EndIf
 If *m\flags=#PB_Image_Border
  p=1
 Else
  p=0
 EndIf
 For n=0 To *m\w-1
   *m\table[n]=*m\table[n+1]
 Next
 *m\table[*m\w-1]=state
 If StartDrawing(ImageOutput(*m\image))
  Box(p,p,*m\w,*m\h,*m\bgcolor)
  For n=0 To *m\h Step 10
    LineXY(p,n,*m\w,n,*m\fgcolor)
  Next
  For n=0 To *m\w+*m\offset Step 10
    LineXY(n-*m\offset,p,n-*m\offset,*m\h,*m\fgcolor)
  Next
  For n=0 To *m\w
    bottom=*m\h
    percent=(*m\table[n]/100.0)
    h=Int(percent * *m\h)
    y1=bottom-h
    percent=(*m\table[n+1]/100)
    h=Int(percent * *m\h)
    y2.l=bottom-h
    LineXY(n+1,y1,n+1,y2,*m\linecolor)
  Next
  *m\offset+1
  StopDrawing()
  SetGadgetState(gadget,*m\imageid)
 EndIf
EndProcedure

;Use this instead of FreeGadget() to avoid memory leaks.
Procedure FreeMeterGadget(gadget.l)
 Protected *m.MeterGadget_struct
 If IsGadget(gadget)
  *m=GetGadgetData(gadget)
  FreeGadget(gadget)
  If *m
   If IsImage(*m\image)
    FreeImage(*m\image)
   EndIf
   FreeMemory(*m)
  EndIf
 EndIf
EndProcedure

;Use this instead of SetGadgetData().
Procedure SetMeterGadgetData(gadget.l,userdata.l)
 Protected *m.MeterGadget_struct
 *m=GetGadgetData(gadget)
 *m\userdata=userdata
EndProcedure

;Use this instead of GetGadgetData().
Procedure.l GetMeterGadgetData(gadget.l)
 Protected *m.MeterGadget_struct
 *m=GetGadgetData(gadget)
 ProcedureReturn *m\userdata
EndProcedure

;Use this instead of GetGadgetState(). Not that usefull but may be handy to have.
Procedure.l GetMeterGadgetState(gadget.l)
 Protected *m.MeterGadget_struct
 *m=GetGadgetData(gadget)
 ProcedureReturn *m\offset
EndProcedure

;Use this to get #Image
Procedure.l GetMeterGadgetImage(gadget.l)
 Protected *m.MeterGadget_struct,result.l=#Null
 *m=GetGadgetData(gadget)
 ProcedureReturn *m\image
EndProcedure

; IDE Options = PureBasic 4.31 (Windows - x86)
; CursorPosition = 30
; Folding = --
; EnableXP
; CurrentDirectory = F:\120GB\WInfo.exe\
; CompileSourceDirectory